from rest_framework import serializers

from generic_permissions.validation import ValidatorMixin

from . import models


class TestModel1Serializer(ValidatorMixin, serializers.ModelSerializer):
    class Meta:
        model = models.Model1
        fields = "__all__"


class TestModel2Serializer(ValidatorMixin, serializers.ModelSerializer):
    class Meta:
        model = models.Model2
        fields = "__all__"



from collections import namedtuple

import pytest
from django.apps import apps
from rest_framework.test import APIClient

User = namedtuple("User", "username")


@pytest.fixture
def user(settings):
    return User(username="user")


@pytest.fixture
def admin_user(settings):
    return User(username="admin")


@pytest.fixture
def client(user):
    client = APIClient()
    client.force_authenticate(user=user)
    return client


@pytest.fixture
def admin_client(admin_user):
    client = APIClient()
    client.force_authenticate(user=admin_user)
    return client


@pytest.fixture(autouse=True)
def reset_config_classes(settings):
    """Reset the config classes to clean state after test.

    The config classes need to be reset after running tests that
    use them. Otherwise, unrelated tests may get affected.
    """

    # First, set config to original value
    core_config = apps.get_app_config("generic_permissions")
    core_config.ready()



from warnings import warn

from .config import ValidatorsConfig

"""
Base for custom validations.

Extend this to implement your custom validators. The validation
methods will receive the data to be validated, and are expected to
return the data back to the caller.

You can modify the data, or raise a `ValidationError` if needed.

Additionally, you can access the request via `self.context['request'].`

Example:
>>> from generic_permissions.validation import BaseValidator, validator_for
>>> from my_project.my_app.models import User
>>> class LowercaseUsername:
...     @validator_for(User)
...     def lowercase_username(self, data):
...         data['username'] = data['username'].lower()
...         return data
"""

validator_for = ValidatorsConfig.decorator


class BaseValidator:
    def __init__(self, *args, **kwargs):  # pragma: no cover
        warn(
            DeprecationWarning(
                "BaseValidator is not required anymore. Just use "
                "a regular class without inheriting from BaseValidator"
            )
        )


class ValidatorMixin:
    def validate(self, *args, **kwargs):
        validated_data = super().validate(*args, **kwargs)

        for method in ValidatorsConfig.get_handlers(
            self.Meta.model,
        ):
            validated_data = method(validated_data, context=self.context)
        return validated_data



from warnings import warn

from django.core.exceptions import PermissionDenied
from django.http import HttpResponse

from .config import ObjectPermissionsConfig, PermissionsConfig

permission_for = PermissionsConfig.decorator
object_permission_for = ObjectPermissionsConfig.decorator


class PermissionViewMixin:
    def destroy(self, request, *args, **kwargs):
        self._check_permissions(request)
        instance = self.get_object()

        # we do not call `super()` in order to not fetch the object twice.
        self.perform_destroy(instance)
        return HttpResponse(status=204)

    def create(self, request, *args, **kwargs):
        self._check_permissions(request)
        return super().create(request, *args, **kwargs)

    def update(self, request, *args, **kwargs):
        self._check_permissions(request)
        return super().update(request, *args, **kwargs)

    def _check_permissions(self, request):
        for handler in PermissionsConfig.get_handlers(self.get_serializer().Meta.model):
            if not handler(request):
                raise PermissionDenied()

    def check_object_permissions(self, request, instance):
        """Check if access to given object is granted.

        Raise PermissionDenied if configured permissions do not allow
        accesss to the object.

        Called by get_object().
        """
        for handler in ObjectPermissionsConfig.get_handlers(
            self.get_serializer().Meta.model
        ):
            if not handler(request, instance):
                raise PermissionDenied()


class BasePermission:
    def __init__(self, *args, **kwargs):  # pragma: no cover
        super().__init__(*args, **kwargs)
        warn(
            DeprecationWarning(
                "BasePermission is not required anymore. Just use "
                "a regular class without inheriting from BasePermission"
            )
        )


class AllowAny:
    @permission_for(object)
    def default_permission(self, request):
        return True

    @object_permission_for(object)
    def default_object_permission(self, request, instance):
        return True


class DenyAll:
    @permission_for(object)
    def default_permission(self, request):
        return False

    @object_permission_for(object)
    def default_object_permission(self, request, instance):
        return False



import inspect
from collections import defaultdict
from functools import wraps

from django.core.exceptions import ImproperlyConfigured


class MethodDecorator:
    def __init__(self, purpose_attr_name):
        self._attr_name = purpose_attr_name

    def __call__(self, model_cls):
        """Decorate custom validator method.

        Decorate your validator methods to mark which
        model they can be used for.
        """

        def _add_dgap_decoration(func):
            attr = getattr(func, self._attr_name, None)
            if attr:
                attr.append(model_cls)
                return func

            @wraps(func)
            def wrapped(*args, **kwargs):
                return func(*args, **kwargs)

            setattr(wrapped, self._attr_name, [model_cls])

            return wrapped

        return _add_dgap_decoration


class DGAPConfigManager:
    def __init__(self, purpose):
        self._purpose = purpose
        # Handlers map from model class -> list-of-handlers
        self._handlers = defaultdict(list)
        self._purpose_attr = f"_dgap_handler_for_{self._purpose}"

        self.decorator = MethodDecorator(self._purpose_attr)

    def get_handlers(self, model_cls, *args, **kwargs):
        """Return a list of callables to handle the given model.

        Additional args and kwargs can be passed - those will be used to
        initialize the handler class.
        """

        # There may be fallback handlers defined on a handler class.
        # We should only return the most specific handler for a given
        # model. The MRO list returns the "lowest", most-specific class
        # first, so we reverse the list, then use a dict to only keep
        # the last one (which will be the most-specific implementation
        # for the given model)

        all_available_lookup_classes = reversed(
            [
                lookup_cls
                for lookup_cls in model_cls.mro()
                if lookup_cls in self._handlers
            ]
        )

        handler_cls_and_method = {
            handler_cls: func_name
            for cls_to_handle in all_available_lookup_classes
            for handler_cls, func_name in self._handlers[cls_to_handle]
        }

        return [
            getattr(handler_cls(*args, **kwargs), func_name)
            for handler_cls, func_name in handler_cls_and_method.items()
        ]

    def register_handler_class(self, handler_cls):
        perm_fns = self.marked_methods(handler_cls)

        seen_models = set()
        for func_name, models in perm_fns:
            for model in models:
                if model in seen_models:
                    raise ImproperlyConfigured(
                        f"{handler_cls.__name__} has multiple methods "
                        f"registered for {self._purpose} on {model.__name__}. "
                        f"Last seen was {func_name}"
                    )
                seen_models.add(model)
                self._handlers[model].append((handler_cls, func_name))

    def _purpose_hasattr(self, handler):
        return hasattr(handler, self._purpose_attr)

    def _purpose_getattr(self, handler, default=None):
        return getattr(handler, self._purpose_attr, default)

    def marked_methods(self, handler_cls):
        return [
            (name, self._purpose_getattr(func))
            for name, func in inspect.getmembers(handler_cls, self._purpose_hasattr)
        ]

    def clear_handlers(self):
        """Clear model-specific handlers on this specific class.

        For instance, `ValidatorMixin.clear()` will clear all handlers of the
        `ValidatorMixin`, but not the ones of the `PermissionMixin` etc.
        """
        self._handlers = defaultdict(list)


PermissionsConfig = DGAPConfigManager(purpose="permission")
ObjectPermissionsConfig = DGAPConfigManager(purpose="object_permission")
VisibilitiesConfig = DGAPConfigManager(purpose="visibility")
ValidatorsConfig = DGAPConfigManager(purpose="validator")



import pytest
from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse
from rest_framework.status import HTTP_200_OK, HTTP_404_NOT_FOUND

from generic_permissions.config import VisibilitiesConfig
from generic_permissions.visibilities import Union, filter_queryset_for

from .models import BaseModel, Model1, Model2


@pytest.mark.parametrize("detail", [True, False])
@pytest.mark.parametrize("use_admin_client", [True, False])
def test_visibility(
    db,
    admin_user,
    admin_client,
    client,
    detail,
    use_admin_client,
):
    client = admin_client if use_admin_client else client

    class TestVisibility:
        @filter_queryset_for(Model1)
        def filter_queryset_for_document(self, queryset, request):
            if request.user.username != "admin":
                return queryset.none()
            return queryset.exclude(text="bar")

    VisibilitiesConfig.register_handler_class(TestVisibility)

    model1 = Model1.objects.create(text="foo")
    Model1.objects.create(text="bar")

    url = reverse("model1-list")
    if detail:
        url = reverse("model1-detail", args=[model1.pk])
    response = client.get(url)
    if detail and not use_admin_client:
        assert response.status_code == HTTP_404_NOT_FOUND
        return
    assert response.status_code == HTTP_200_OK
    result = response.json()
    if not detail:
        if use_admin_client:
            assert len(result) == 1
            assert result[0]["text"] == "foo"
        else:
            assert len(result) == 0
    else:
        assert result["text"] == "foo"


def test_visibility_dupes(client):
    class TestVisibility:
        @filter_queryset_for(Model1)
        def filter_queryset_for_document(self, queryset, request):  # pragma: no cover
            return queryset

        @filter_queryset_for(Model1)
        def filter_queryset_for_document2(self, queryset, request):  # pragma: no cover
            return queryset

    with pytest.raises(ImproperlyConfigured):
        VisibilitiesConfig.register_handler_class(TestVisibility)


def test_custom_visibility_for_basemodel(db, client):
    """Test fallback to BaseModel."""
    Model1.objects.create(text="m1")
    Model1.objects.create(text="m2")

    class CustomVisibility:
        @filter_queryset_for(BaseModel)
        def filter_queryset_for_all(self, queryset, request):
            return queryset.none()

    VisibilitiesConfig.register_handler_class(CustomVisibility)

    assert Model1.objects.count() == 2

    url = reverse("model1-list")
    response = client.get(url)
    assert response.status_code == HTTP_200_OK
    result = response.json()

    assert result == []


def test_custom_visibility_override_specificity(db, admin_client):
    """The first matching filter 'wins'."""
    Model1.objects.create(text="m1")
    Model1.objects.create(text="m2")

    class CustomVisibility:
        @filter_queryset_for(BaseModel)
        def filter_queryset_for_all(self, queryset, request):
            return queryset.none()  # pragma: no cover

        @filter_queryset_for(Model1)
        def filter_queryset_for_document(self, queryset, request):
            return queryset.filter(text="m1")

    assert Model1.objects.count() == 2

    VisibilitiesConfig.clear_handlers()
    VisibilitiesConfig.register_handler_class(CustomVisibility)

    assert len(admin_client.get(reverse("model1-list")).json()) == 1


def test_custom_visibility_chained_decorators(admin_client, db):
    class CustomVisibility:
        @filter_queryset_for(BaseModel)
        def filter_queryset_for_all(self, queryset, request):
            return queryset.none()  # pragma: no cover

        @filter_queryset_for(Model1)
        @filter_queryset_for(Model2)
        def filter_queryset_for_custom_node(self, queryset, request):
            return queryset.filter(text="name1")

    Model1.objects.create(text="name1")
    Model1.objects.create(text="name2")
    Model2.objects.create(text="name1")
    Model2.objects.create(text="name2")

    assert Model1.objects.count() == 2
    assert Model2.objects.count() == 2

    VisibilitiesConfig.clear_handlers()
    VisibilitiesConfig.register_handler_class(CustomVisibility)

    assert len(admin_client.get(reverse("model1-list")).json()) == 1
    assert len(admin_client.get(reverse("model2-list")).json()) == 1


def test_union_visibility(db, admin_client):
    Model1.objects.create(text="m1")
    Model1.objects.create(text="m2")
    Model1.objects.create(text="m3")
    Model1.objects.create(text="m4")

    class Name1Visibility:
        @filter_queryset_for(Model1)
        def filter_queryset_for_custom_node(self, queryset, request):
            return queryset.filter(text="m1")

    class Name2Visibility:
        @filter_queryset_for(Model1)
        def filter_queryset_for_custom_node(self, queryset, request):
            return queryset.filter(text="m2")

    class Name3Visibility:
        @filter_queryset_for(Model1)
        def filter_queryset_for_custom_node(self, queryset, request):
            return queryset.filter(text__in=["m2", "m3"])

    class ConfiguredUnion(Union):
        visibility_classes = [Name1Visibility, Name2Visibility, Name3Visibility]

    url = reverse("model1-list")

    VisibilitiesConfig.clear_handlers()
    VisibilitiesConfig.register_handler_class(Name1Visibility)
    assert len(admin_client.get(url).json()) == 1

    VisibilitiesConfig.clear_handlers()
    VisibilitiesConfig.register_handler_class(Name2Visibility)
    assert len(admin_client.get(url).json()) == 1

    VisibilitiesConfig.clear_handlers()
    VisibilitiesConfig.register_handler_class(Name3Visibility)
    assert len(admin_client.get(url).json()) == 2

    VisibilitiesConfig.clear_handlers()
    VisibilitiesConfig.register_handler_class(ConfiguredUnion)
    assert len(admin_client.get(url).json()) == 3


def test_union_visibility_none(db, admin_client):
    Model1.objects.create(text="m1")

    class CustomVisibility:
        @filter_queryset_for(Model1)
        def filter_queryset_for_custom_node(self, queryset, request):
            return queryset.none()

    class CustomVisibility2:
        @filter_queryset_for(Model1)
        def filter_queryset_for_custom_node(self, queryset, request):
            return queryset.none()

    class ConfiguredUnion(Union):
        visibility_classes = [CustomVisibility2, CustomVisibility]

    url = reverse("model1-list")

    VisibilitiesConfig.clear_handlers()
    VisibilitiesConfig.register_handler_class(CustomVisibility)
    assert len(admin_client.get(url).json()) == 0

    VisibilitiesConfig.clear_handlers()
    VisibilitiesConfig.register_handler_class(CustomVisibility2)
    assert len(admin_client.get(url).json()) == 0

    VisibilitiesConfig.clear_handlers()
    VisibilitiesConfig.register_handler_class(ConfiguredUnion)
    assert len(admin_client.get(url).json()) == 0



import pytest
from django.core.exceptions import ImproperlyConfigured, PermissionDenied
from django.urls import reverse
from rest_framework.status import (
    HTTP_200_OK,
    HTTP_201_CREATED,
    HTTP_204_NO_CONTENT,
    HTTP_403_FORBIDDEN,
)

from generic_permissions.config import ObjectPermissionsConfig, PermissionsConfig
from generic_permissions.permissions import (
    DenyAll,
    object_permission_for,
    permission_for,
)
from tests.views import Test1ViewSet, Test2ViewSet

from .models import Model1, Model2


@pytest.mark.parametrize(
    "method,status",
    [
        ("post", HTTP_201_CREATED),
        ("patch", HTTP_200_OK),
        ("delete", HTTP_204_NO_CONTENT),
    ],
)
@pytest.mark.parametrize("use_admin_client", [True, False])
def test_permission(
    db,
    admin_client,
    client,
    method,
    status,
    use_admin_client,
):
    client = admin_client if use_admin_client else client

    class CustomPermission:
        @permission_for(Model1)
        def has_permission_for_document(self, request):
            if (
                hasattr(request, "user")
                and request.user.username == "admin"
                or request.data["text"] == "baz"
            ):
                return True
            return False

        @object_permission_for(Model1)
        def has_object_permission_for_document(self, request, instance):
            assert isinstance(instance, Model1)
            if hasattr(request, "user") and request.user.username == "admin":
                return True
            return False

    PermissionsConfig.register_handler_class(CustomPermission)
    ObjectPermissionsConfig.register_handler_class(CustomPermission)

    tm = Model1.objects.create(text="foo")

    url = reverse("model1-list")

    if method in ["patch", "delete"]:
        url = reverse("model1-detail", args=[tm.pk])

    data = {"text": "bar"}
    if method == "patch":
        data = {"text": "baz"}

    response = getattr(client, method)(url, data=data)

    if not use_admin_client:
        assert response.status_code == HTTP_403_FORBIDDEN
        return

    assert response.status_code == status

    if method == "post":
        assert response.json()["text"] == "bar"
        Model1.objects.get(text="bar")
    elif method == "patch":
        assert response.json()["text"] == "baz"
        tm.refresh_from_db()
        assert tm.text == "baz"


def test_custom_permission_override_has_permission_with_duplicates():
    class CustomPermission:
        @permission_for(Model1)
        def has_permission_for_custom_mutation(self, request):  # pragma: no cover
            return False

        @permission_for(Model1)
        def has_permission_for_custom_mutation_2(self, request):  # pragma: no cover
            return False

    with pytest.raises(ImproperlyConfigured):
        PermissionsConfig.register_handler_class(CustomPermission)


def test_custom_permission_override_has_object_permission_with_duplicates():
    class CustomPermission:
        @object_permission_for(Model1)
        def has_object_permission_for_custom_mutation(
            self, request, instance
        ):  # pragma: no cover
            return False

        @object_permission_for(Model1)
        def has_object_permission_for_custom_mutation_2(
            self, request, instance
        ):  # pragma: no cover
            return False

    with pytest.raises(ImproperlyConfigured):
        ObjectPermissionsConfig.register_handler_class(CustomPermission)


def test_custom_permission_override_has_permission_with_multiple_models(admin_client):
    class CustomPermission:
        @permission_for(Model1)
        @permission_for(Model2)
        def has_permission_for_both_mutations(self, request):
            return False

    PermissionsConfig.register_handler_class(CustomPermission)

    for model in Model1, Model2:
        url = reverse(f"{model.__name__.lower()}-list")
        resp = admin_client.post(url, data={"text": "foo"}, format="json")
        assert resp.status_code == HTTP_403_FORBIDDEN


def test_custom_permission_override_has_object_permission_with_multiple_mutations(
    db, admin_client
):
    class CustomPermission:
        @object_permission_for(Model1)
        def has_object_permission_for_both_mutations(self, request, instance):
            return False

    ObjectPermissionsConfig.register_handler_class(CustomPermission)

    tm1 = Model1.objects.create()

    url = reverse("model1-detail", args=[tm1.pk])
    resp = admin_client.patch(url, data={"text": "foo"}, format="json")
    assert resp.status_code == HTTP_403_FORBIDDEN


def test_custom_permission_fallback_to_true(
    db,
    rf,
):
    class CustomPermission:
        @permission_for(Model2)
        def has_permission_for_document(self, request):  # pragma: no cover
            return False

        @object_permission_for(Model2)
        def has_object_permission_for_both_mutations(
            self, request, instance
        ):  # pragma: no cover
            return False

    tm1 = Model1.objects.create()
    ObjectPermissionsConfig.register_handler_class(CustomPermission)
    PermissionsConfig.register_handler_class(CustomPermission)

    request = rf.patch("")
    request.data = {"text": "foo"}

    Test1ViewSet(request=request, format_kwarg="json")._check_permissions(request)
    Test1ViewSet(request=request, format_kwarg="json").check_object_permissions(
        request, tm1
    )


def test_deny_all_permission(
    db,
    rf,
):
    class CustomPermission(DenyAll):
        @permission_for(Model2)
        def has_permission_for_document(self, request):
            return True

        @object_permission_for(Model2)
        def has_object_permission_for_both_mutations(self, request, instance):
            return True

    tm1 = Model1.objects.create()
    tm2 = Model2.objects.create()
    ObjectPermissionsConfig.register_handler_class(CustomPermission)
    PermissionsConfig.register_handler_class(CustomPermission)

    request = rf.patch("")
    request.data = {"text": "foo"}

    # Model2 access should be granted by above permission class
    Test2ViewSet(request=request, format_kwarg="json")._check_permissions(request)
    Test2ViewSet(request=request, format_kwarg="json").check_object_permissions(
        request, tm2
    )

    # Model1 access should "bubble up" to the DenyAll base class and be denied
    with pytest.raises(PermissionDenied):
        Test1ViewSet(request=request, format_kwarg="json")._check_permissions(request)

    with pytest.raises(PermissionDenied):
        Test1ViewSet(request=request, format_kwarg="json").check_object_permissions(
            request, tm1
        )



from functools import reduce
from warnings import warn

from .config import DGAPConfigManager, VisibilitiesConfig

"""
Basic visibility classes to be extended by any visibility implementation.

In combination with the decorator `@filter_queryset_for` a custom
visibility class can define filtering on basis of models.

A custom visibility class could look like this:
```
>>> from generic_permissions.visibilities import BaseVisibility
... from tests.models import BaseModel, Model1, Model2
...
...
... class CustomVisibility:
...     @filter_queryset_for(BaseModel)
...     def filter_queryset_for_all(self, queryset, request):
...         return queryset.filter(created_by_user=request.user.username)
...
...     @filter_queryset_for(Model1)
...     def filter_queryset_for_document(self, queryset, request):
...         return queryset.exclude(category__slug='protected-category')
...
...     @filter_queryset_for(Model2)
...     def filter_queryset_for_file(self, queryset, request):
...         # Limitations for `Model1` should also be enforced on `Model2`.
...         return queryset.exclude(document__category__slug='protected-category')
```
"""


filter_queryset_for = VisibilitiesConfig.decorator


class VisibilityViewMixin:
    def get_queryset(self):
        queryset = super().get_queryset()
        for handler in VisibilitiesConfig.get_handlers(queryset.model):
            queryset = handler(queryset, self.request)

        return queryset


class BaseVisibility:  # pragma: no cover
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        warn(
            DeprecationWarning(
                "BaseVisibility is not required anymore. Just use "
                "a regular class without inheriting from BaseVisibility"
            )
        )


class Any:
    """No restrictions, all models are exposed."""

    pass


class Union:
    """Union result of a list of configured visibility classes."""

    visibility_classes = []

    def __init__(self):
        self._config = DGAPConfigManager("visibility")
        for cls in self.visibility_classes:
            self._config.register_handler_class(cls)

    @filter_queryset_for(object)
    def filter_queryset(self, queryset, request):
        handlers = self._config.get_handlers(queryset.model)
        querysets_to_merge = [handler(queryset, request) for handler in handlers]

        union_qs = reduce(lambda a, b: a.union(b), querysets_to_merge, queryset.none())

        return queryset.filter(pk__in=union_qs.values("pk"))



from django.urls import reverse
from rest_framework.exceptions import ValidationError

from generic_permissions.config import ValidatorsConfig
from generic_permissions.validation import validator_for

from .models import Model1, Model2


def test_custom_validation(db, admin_client):
    model1 = Model1.objects.create(text="hello")

    class LowercaseUsername:
        @validator_for(Model1)
        def lowercase_text_attr(self, data, context):
            data["text"] = data["text"].lower()
            return data

    class RejectEverything:
        # Two validator markers - we support stacking/chaining
        @validator_for(Model1)
        @validator_for(Model2)
        def reject_all(self, data, context):
            raise ValidationError("NOPE")

    def send_update():
        return admin_client.patch(
            url,
            format="json",
            data={
                "text": "SOME UPPERCASE TEXT",
                "id": str(model1.pk),
            },
        )

    url = reverse("model1-detail", args=[model1.pk])

    # First, check if rejection works
    ValidatorsConfig.clear_handlers()
    ValidatorsConfig.register_handler_class(RejectEverything)
    resp = send_update()

    assert resp.json() == {"non_field_errors": ["NOPE"]}

    # Then, check if the data modification works
    ValidatorsConfig.clear_handlers()
    ValidatorsConfig.register_handler_class(LowercaseUsername)

    resp = send_update()
    assert resp.json()["text"] == "some uppercase text"

    # Also check in DB
    model1.refresh_from_db()
    assert model1.text == "some uppercase text"



from warnings import warn


class VisibilityModelMixin:
    def __init__(self, *args, **kwargs):  # pragma: no cover
        super().__init__(*args, **kwargs)
        warn(
            DeprecationWarning(
                "VisibilityModelMixin is not required anymore and "
                "will be removed in a future version of DGAP"
            )
        )


class PermissionModelMixin:
    def __init__(self, *args, **kwargs):  # pragma: no cover
        super().__init__(*args, **kwargs)
        warn(
            DeprecationWarning(
                "PermissionModelMixin is not required anymore and "
                "will be removed in a future version of DGAP"
            )
        )



DATABASES = {
    "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"},
}

INSTALLED_APPS = (
    "tests",
    "generic_permissions.apps.GenericPermissionsConfig",
)

GENERIC_PERMISSIONS_PERMISSION_CLASSES = ["generic_permissions.permissions.AllowAny"]
GENERIC_PERMISSIONS_VISIBILITY_CLASSES = ["generic_permissions.visibilities.Any"]
GENERIC_PERMISSIONS_VALIDATION_CLASSES = ["generic_permissions.validations.PassThrough"]

ROOT_URLCONF = "tests.urls"

SECRET_KEY = "foobar"
USE_TZ = True



# Generated by Django 3.1.4 on 2020-12-17 14:42

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Model1",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "text",
                    models.CharField(
                        help_text="Text description.",
                        max_length=100,
                        verbose_name="Text comes here",
                    ),
                ),
            ],
            options={"abstract": False},
        ),
        migrations.CreateModel(
            name="Model2",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "text",
                    models.CharField(
                        help_text="Text description.",
                        max_length=100,
                        verbose_name="Text comes here",
                    ),
                ),
            ],
            options={"abstract": False},
        ),
    ]



from warnings import warn  # pragma: no cover


class PermissionSerializerMixin:  # pragma: no cover
    def __init__(self, *args, **kwargs):
        warn(
            DeprecationWarning(
                "PermissionSerializerMixin is not required anymore. You "
                "only need to decorate your viewsets now, using "
                "generic_permissions.permissions.PermissionViewMixin."
            )
        )
        super().__init__(*args, **kwargs)



# -*- coding: utf-8 -*-

from codecs import open
from os import path

from setuptools import setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="django-generic-api-permissions",
    version="0.2.0",
    author="adfinis",
    description="Generic API permissions and visibilities for Django",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/adfinis-sygroup/django-generic-api-permissions",
    license="License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
    packages=["generic_permissions"],
    install_requires=["django>=2.2"],
    python_requires=">=3.5",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Environment :: Web Environment",
        "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



from rest_framework.viewsets import ModelViewSet

from generic_permissions.permissions import PermissionViewMixin
from generic_permissions.visibilities import VisibilityViewMixin

from . import models, serializers


class Test1ViewSet(PermissionViewMixin, VisibilityViewMixin, ModelViewSet):
    serializer_class = serializers.TestModel1Serializer
    queryset = models.Model1.objects.all()


class Test2ViewSet(PermissionViewMixin, VisibilityViewMixin, ModelViewSet):
    serializer_class = serializers.TestModel2Serializer
    queryset = models.Model2.objects.all()



from warnings import warn  # pragma: no cover

from generic_permissions import permissions, visibilities  # pragma: no cover


class VisibilityViewMixin(visibilities.VisibilityViewMixin):  # pragma: no cover
    def __init__(self, *args, **kwargs):
        warn(
            DeprecationWarning(
                "VisibilityViewMixin has moved from generic_permissions.views "
                "to generic_permissions.visibilities. Please update your imports"
            )
        )
        super().__init__(*args, **kwargs)


class PermissionViewMixin(permissions.PermissionViewMixin):  # pragma: no cover
    def __init__(self, *args, **kwargs):
        warn(
            DeprecationWarning(
                "PermissionViewMixin has moved from generic_permissions.views "
                "to generic_permissions.permissions. Please update your imports"
            )
        )
        super().__init__(*args, **kwargs)



from django.apps import AppConfig
from django.conf import settings
from django.utils.module_loading import import_string

from generic_permissions.config import (
    ObjectPermissionsConfig,
    PermissionsConfig,
    ValidatorsConfig,
    VisibilitiesConfig,
)
from generic_permissions.permissions import AllowAny
from generic_permissions.visibilities import Any


class GenericPermissionsConfig(AppConfig):
    name = "generic_permissions"
    verbose_name = "Django Generic API Permissions"

    def ready(self):
        self._init_config(
            "GENERIC_PERMISSIONS_PERMISSION_CLASSES",
            [AllowAny],
            [ObjectPermissionsConfig, PermissionsConfig],
        )

        self._init_config(
            "GENERIC_PERMISSIONS_VISIBILITY_CLASSES", [Any], [VisibilitiesConfig]
        )

        self._init_config(
            "GENERIC_PERMISSIONS_VALIDATOR_CLASSES", [], [ValidatorsConfig]
        )

    def _init_config(self, setting_name, defaults, configs):
        classes = defaults
        classes_strings = getattr(settings, setting_name, None)
        if classes_strings:
            classes = [import_string(name) for name in classes_strings]
        for c in configs:
            c.clear_handlers()
            for handler in classes:
                c.register_handler_class(handler)



from django.db import models
from django.utils.translation import gettext_lazy as _


class BaseModel(models.Model):
    """Base for test models that sets app_label, so they play nicely."""

    class Meta:
        app_label = "tests"
        abstract = True


class Model1(BaseModel):
    text = models.CharField(
        max_length=100,
        verbose_name=_("Text comes here"),
        help_text=_("Text description."),
    )


class Model2(BaseModel):
    text = models.CharField(
        max_length=100,
        verbose_name=_("Text comes here"),
        help_text=_("Text description."),
    )
